package eu.pequignat.function;

import eu.pequignat.MyFunction;
import eu.pequignat.runnable.MyCallable;

import java.util.Iterator;
import java.util.List;
import java.util.Queue;
import java.util.concurrent.*;

import static eu.pequignat.constant.Constant.NB_THREAD_IN_PARALLEL;

/**
 * This exemple use a better implemetation of fixed number of Thread.
 * Used the class ExecutorService with Executors.newFixedThreadPool(App.NB_THREAD_IN_PARALLEL);
 */
public class MultiThreadingFixedPoolExecutor implements MyFunction {
    @Override
    public void run() {
        ExecutorService executorService = Executors.newFixedThreadPool(NB_THREAD_IN_PARALLEL);
        Queue<Callable<String>> allCallableQueue = new ConcurrentLinkedQueue<>();

        message("Prepare the List of all Thread to start in parallel");
        for (int i = 0; i< NB_THREAD_IN_PARALLEL; i++){
            allCallableQueue.add(new MyCallable());
        }
        message("start all Thread");
        try {
            List<Future<String>> futures =  executorService.invokeAll(allCallableQueue);
            message("All are terminated");
            for (Future<String> f : futures){
                try {
                    message(this.toString().concat(": ").concat(f.get()));
                } catch (ExecutionException e) {
                    LOG.error(e.getLocalizedMessage(),e);
                }
            }
            message("We need to shutdown the ExecutorService when all Futures are Done");
            executorService.shutdown();

        } catch (InterruptedException e) {
            LOG.error(e.getLocalizedMessage(),e);
        }
    }
}
