package eu.pequignat.problem.pc;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.LinkedList;
import java.util.concurrent.atomic.AtomicBoolean;

import static eu.pequignat.constant.Constant.SLEEP_MS;

public class QueuePC {

    private static Logger LOG = LoggerFactory.getLogger(QueuePC.class);
    private int capacity = 2;

    // Create a list shared by producer and consumer
    // Size of list is 2.
    private LinkedList<Integer> list = new LinkedList<>();

    private final AtomicBoolean stopBool= new AtomicBoolean(Boolean.FALSE);

    // Function called by producer thread
    public void produce() throws InterruptedException
    {
        int value = 0;
        while (! stopBool.get()) {
            synchronized (this) {
                if (list.size() < capacity) {
                    LOG.info("Producer produced-" + value);
                    // to insert the jobs in the list
                    list.add(value++);

                    // notifies the consumer thread that
                    // now it can start consuming
                    notify();

                    // makes the working of program easier
                    // to  understand
                    Thread.sleep(SLEEP_MS/2);
                } else {
                    wait();
                }
            }
        }
    }

    // Function called by consumer thread
    public void consume() throws InterruptedException
    {
        while (!stopBool.get()) {
            synchronized (this) {
                if (list.size() > 0){
                    // to retrieve the first job in the list
                    int val = list.removeFirst();
                    LOG.info("Consumer consumed-" + val);

                    // Wake up producer thread
                    notify();
                    // and sleep
                    Thread.sleep(SLEEP_MS);
                }else{
                    wait();
                }
            }
        }
    }

    public void stop(){
        LOG.info("Stop called");
        stopBool.set(Boolean.TRUE);
    }

}
